using System;
using System.Runtime.InteropServices;
using System.Threading;
using Westwind.InternetTools;
using Westwind.Tools;

namespace DotNetCOM.WinFormSample
{
	/// <summary>
	/// Summary description for DownloadUrl.
	/// </summary>
	[ProgId("DotNetCom.DownloadForm")]
	[ClassInterface(ClassInterfaceType.AutoDual)]
	[ComSourceInterfaces( typeof(IDownloadFormEvents) )]
	public class DownloadForm : Westwind.Windows.Dialogs.ProgressForm
	{
		string Url
		{
			get
			{
				return _Url;
			}
			set
			{
				_Url = value;
			}
		}
		string _Url = "";

		public string Mesage
		{
			get
			{
				return _Message;
			}
			set
			{
				_Message= value;
			}
		}
		string _Message = "Downloading...";
	
		public int BufferSize
		{
			get
			{
				return _BufferSize;
			}
			set
			{
				_BufferSize= value;
			}
		}
		int _BufferSize = 4096;

		public byte[] ResponseData
		{
			get
			{
				return _ResponseData;
			}
			set
			{
				_ResponseData = value;
			}
		}
		byte[] _ResponseData = null;

		private wwHttp Http = null;
		private DownloadForm Instance = null;

		public DownloadForm()
		{
		}

		public DownloadForm DownloadFormFactory() 
		{
			Instance = new DownloadForm();
			Thread th = new Thread( new ThreadStart(this.StartForm));
			th.Start();

			return Instance;
		}

		private void StartForm() 
		{
			System.Windows.Forms.Application.EnableVisualStyles();
			System.Windows.Forms.Application.Run();
		}

		public void StartDownload(string Url, string Header) 
		{
			this.Show();
			

			this.Url = Url;

			Thread th = new Thread( new ThreadStart(this.RunDownload));
			th.Start();
		}

		private void RunDownload()
		{
			try 
			{
				this.Http = new wwHttp();
				this.Http.ReceiveData +=new Westwind.InternetTools.wwHttp.ReceiveDataDelegate(Http_ReceiveData);
				this.UpdateStatus("Downloading from " + wwUtils.ExtractString(this.Url,"http://","/") ,-1);
				this. ResponseData = this.Http.GetUrlBytes(this.Url,this.BufferSize);
				this.UpdateStatus(100);
			}
			catch(Exception ex) 
			{
				this.ErrorMessage = ex.Message;
				this.Cancelled = true;
				if (this.DownloadCancelled != null)
					this.DownloadCancelled();
			}
			if (!this.Cancelled)
			{
				this.Visible = false;
				if (this.DownloadComplete != null) 
					this.DownloadComplete();
			}
		}
		
		public new void Cancel() 
		{
			// *** Don't cancel immediately so the event fires
			// *** first.
			this.Cancelled = true;
		}

		private void Http_ReceiveData(object sender, Westwind.InternetTools.ReceiveDataEventArgs e)
		{
			// *** System.Windows.Forms.Application.DoEvents();

			if (e.Cancel)
			{
					this.UpdateStatus("Download cancelled.",100);
					if (this.DownloadCancelled != null)
						this.DownloadCancelled();
			}
			else if(e.Done)
			{
				this.UpdateStatus("Download completed.",100);
				System.Threading.Thread.Sleep(1000);  
			}
			else
			{
				double Temp =  (double)e.CurrentByteCount/(double) e.TotalBytes  * 100.00F;
				this.UpdateStatus("Downloading from " + 
					Westwind.Tools.wwUtils.ExtractString(this.Url,"://","/"),
					Convert.ToInt32(Temp) );
			}
			
			e.Cancel = this.Cancelled;
			
			this.Refresh();
		}

		public delegate void delDownloadEvent();

		public event delDownloadEvent DownloadComplete;
		public event delDownloadEvent DownloadCancelled;

		private void InitializeComponent()
		{
			// 
			// DownloadForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 14);
			this.ClientSize = new System.Drawing.Size(408, 133);
			this.Name = "DownloadForm";
			this.ShowCancelButton = true;

		}

	}

	/// <summary>
	/// Create the Download Event interface
	/// </summary>
	[InterfaceType(ComInterfaceType.InterfaceIsIDispatch)]
	public interface IDownloadFormEvents 
	{
		[DispId(1)] void DownloadComplete();
		[DispId(2)] void DownloadCancelled();
	}
}
