using System;
using System.Drawing;
using System.Drawing.Imaging;

using System.Runtime.InteropServices;

namespace DotNetCOM
{
	/// <summary>
	/// Summary description for wwImaging.
	/// </summary>
	/// 
	[ClassInterface(ClassInterfaceType.AutoDual)]
	[ProgId("DotNetCOM.wwImaging")]	
	public class wwImaging
	{

		public string cErrorMsg = "";
		public bool bError = false;

		public wwImaging()
		{
		}		

		public void Dispose()
		{
		}

		/// <summary>
		/// Creates a resized bitmap from an existing image on disk. Resizes the image by 
		/// creating an aspect ratio safe image. Image is sized to the larger size of width
		/// height and then smaller size is adjusted by aspect ratio.
		/// 
		/// Image is returned as Bitmap - call Dispose() on the returned Bitmap object
		/// </summary>
		/// <param name="lcFilename"></param>
		/// <param name="lnWidth"></param>
		/// <param name="lnHeight"></param>
		/// <returns>Bitmap or null</returns>
		public Bitmap ResizeImage(string lcFilename,int lnWidth, int lnHeight)
		{
			System.Drawing.Bitmap bmpOut = null;

			try 
			{
				Bitmap loBMP = new Bitmap(lcFilename);
				ImageFormat loFormat = loBMP.RawFormat;


				decimal lnRatio;
				int lnNewWidth = 0;
				int lnNewHeight = 0;

				//*** If the image is smaller than a thumbnail just return it
				if (loBMP.Width < lnWidth && loBMP.Height < lnHeight) 
					return loBMP;
			

				if (loBMP.Width > loBMP.Height)
				{
					lnRatio = (decimal) lnWidth / loBMP.Width;
					lnNewWidth = lnWidth;
					decimal lnTemp = loBMP.Height * lnRatio;
					lnNewHeight = (int)lnTemp;
				}
				else 
				{
					lnRatio = (decimal) lnHeight / loBMP.Height;
					lnNewHeight = lnHeight;
					decimal lnTemp = loBMP.Width * lnRatio;
					lnNewWidth = (int) lnTemp;
				}

				bmpOut = new Bitmap(lnNewWidth, lnNewHeight);
				Graphics g = Graphics.FromImage(bmpOut);
				g.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.HighQualityBicubic;
				g.FillRectangle( Brushes.White,0,0,lnNewWidth,lnNewHeight);
				g.DrawImage(loBMP,0,0,lnNewWidth,lnNewHeight);

				//System.Drawing.Image imgOut = loBMP.GetThumbnailImage(lnNewWidth,lnNewHeight,null,IntPtr.Zero);
				loBMP.Dispose();
			}
			catch 
			{
				return null;
			}
		
			return bmpOut;
		}	

		public bool ResizeImage(string lcFilename, string lcOutputFilename, 
			int lnWidth, int lnHeight)
		{
			System.Drawing.Bitmap bmpOut = null;

			try 
			{
				Bitmap loBMP = new Bitmap(lcFilename);
				ImageFormat loFormat = loBMP.RawFormat;

				decimal lnRatio;
				int lnNewWidth = 0;
				int lnNewHeight = 0;

				//*** If the image is smaller than a thumbnail just return it
				if (loBMP.Width < lnWidth && loBMP.Height < lnHeight) 
				{ 
					loBMP.Save(lcOutputFilename);
					return true;
				}

				if (loBMP.Width > loBMP.Height)
				{
					lnRatio = (decimal) lnWidth / loBMP.Width;
					lnNewWidth = lnWidth;
					decimal lnTemp = loBMP.Height * lnRatio;
					lnNewHeight = (int)lnTemp;
				}
				else 
				{
					lnRatio = (decimal) lnHeight / loBMP.Height;
					lnNewHeight = lnHeight;
					decimal lnTemp = loBMP.Width * lnRatio;
					lnNewWidth = (int) lnTemp;
				}

				bmpOut = new Bitmap(lnNewWidth, lnNewHeight);
				Graphics g = Graphics.FromImage(bmpOut);
				g.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.HighQualityBicubic;
				g.FillRectangle( Brushes.White,0,0,lnNewWidth,lnNewHeight);
				g.DrawImage(loBMP,0,0,lnNewWidth,lnNewHeight);

				//System.Drawing.Image imgOut = loBMP.GetThumbnailImage(lnNewWidth,lnNewHeight,null,IntPtr.Zero);
				loBMP.Dispose();

				bmpOut.Save(lcOutputFilename,loFormat);
				bmpOut.Dispose();
			}
			catch 
			{
				return false;
			}
		
			return true;
		}	


		/// <summary>
		/// Rotates an image file in place.
		/// </summary>
		/// <param name="lcFileName">Filename to rotate.</param>
		/// <param name="lnRotation">1 - 90 degrees, 2 - 180, 3 - 270, 4 - Flip, 5 - Rotate 90 and flip etc.</param>
		/// <returns>True or False</returns>
		public bool RotateImage(string lcFileName, int lnRotation) 
		{
			try 
			{ 
				Bitmap  loImage = new Bitmap(lcFileName);
				loImage.RotateFlip((RotateFlipType) lnRotation);

				Bitmap loNewImage = new Bitmap(loImage.Width,loImage.Height);
				Graphics g = Graphics.FromImage(loNewImage);
				g.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.HighQualityBicubic;
				g.DrawImage(loImage,0,0,loImage.Width,loImage.Height);

				loImage.Dispose();
				loImage = null;

				loNewImage.Save(lcFileName);
			
				return true;
			}
			catch(Exception ex) 
			{
				this.cErrorMsg = ex.Message;
			}

			return false;
		}

		public wwImageInfo GetImageInfo(string lcImageFile)
		{
			try 
			{
				Bitmap loBMP = new Bitmap(lcImageFile);
				
				wwImageInfo loInfo = new wwImageInfo();
				
				loInfo.Width = loBMP.Width;
				loInfo.Height = loBMP.Height;
				loInfo.HorizontalPixelResolution = loBMP.HorizontalResolution;
				loInfo.VerticalPixelResolution = loBMP.VerticalResolution;

				loBMP.Dispose();

				return loInfo;
			}
			catch(Exception ex) 
			{			
				this.SetError(ex.Message); 
			}

			return null;
		}

		public Bitmap GetBitmap(string lcImageFile)
		{
			try 
			{
				Bitmap loBMP = new Bitmap(lcImageFile);
				
				return loBMP;
			}
			catch(Exception ex) 
			{			
				this.SetError(ex.Message); 
			}

			return null;
		}
		
		
		/// <summary>
		///  Sets the error properties of the object
		/// </summary>
		/// <param name="lcError"></param>
		protected void SetError(string lcError) 
		{
			if (lcError.Length == 0)  
			{
				this.cErrorMsg = "";
				this.bError = false;
			}
			else 
			{
				this.cErrorMsg = lcError;		
				this.bError = true;
			}
		}

		
	}

	[ClassInterface(ClassInterfaceType.AutoDual)]
	public class wwImageInfo 
	{
		public int Height = 0;
		public int Width = 0;
		public float HorizontalPixelResolution = 0;
		public float VerticalPixelResolution = 0;
	}
}
