using System;
using System.Runtime.InteropServices;

using System.Reflection;
using System.Xml;

namespace Westwind.wwReflection
{
	/// <summary>
	/// Summary description for DotNetClassInfo.
	/// </summary>
	[ClassInterface(ClassInterfaceType.AutoDual)]
	[ProgId("wwReflection.TypeParser")]
	public class TypeParser
	{
		// ** All X properties are native objects		
		public DotnetObject[] aObjects;
		public Type[] aXObjects;
		public int nObjectCount = 0;
		public string cSyntax = "VB";
 
		/// <summary>
		/// If calling GetObject() this returns a single object to parse
		/// </summary>
		public DotnetObject oObject;

		public string[] aConstants;
		public int nConstantCount = 0;

		public bool lError = false;
		public string cErrorMsg = "";


		public string cFilename = "";
		public string cXmlFilename = "";

		public TypeParser()
		{
			//
			// TODO: Add constructor logic here
			//
		}


		public bool GetObject(string lcClass) 
		{
			if (this.aObjects == null)
			   this.GetAllObjects(true);

			for (int x=0; x< this.aXObjects.Length; x++) 
			{
				Type loType = this.aXObjects[x];
				if (loType.Name.ToLower() != lcClass.ToLower()) 
					continue;

				this.oObject = new DotnetObject();
				this.oObject.cName = loType.Name;
				if (loType.IsPublic)
					this.oObject.cScope = "public";

				else if (loType.IsNotPublic) 
					this.oObject.cScope = "private";

				this.oObject.cNamespace = loType.Namespace;

				this.oObject.LoadMethods(loType);
				this.oObject.LoadProperties(loType);
				this.oObject.LoadEvents(loType);
			}

			if (this.cXmlFilename.Length > 0)  
			{
				this.ParseXmlProperties(this.oObject);
			}
			return true;
		}


		/// <summary>
		/// Gets all objects into the aObjects array and parses all method/properties into the subarrays.
		/// </summary>
		/// <param name="llDontParseMethods">if true methods and properties are not parsed</param>
		/// <returns></returns>
		public int GetAllObjects(bool llDontParseMethods) 
		{
			
			Assembly loAssembly;
			try 
			{
			   loAssembly = Assembly.LoadFrom(this.cFilename);
			}
			catch(Exception ex)
			{
				this.lError = true;
				this.cErrorMsg = ex.Message;
				return 0;
			}

			this.aXObjects = loAssembly.GetTypes();
			int lnCount = this.aXObjects.Length;
            if (lnCount == 0)
				 return 0;

			this.nObjectCount = lnCount;
			this.aObjects = new DotnetObject[lnCount];


			for (int x=0; x< this.aXObjects.Length; x++) 
			{
				Type loType = this.aXObjects[x];

				this.aObjects[x] = new DotnetObject();
				this.aObjects[x].cName = loType.Name;
				this.aObjects[x].cSyntax = this.cSyntax;

				if (loType.IsPublic)
					this.aObjects[x].cScope = "public";
				else if (loType.IsNotPublic) 
					this.aObjects[x].cScope = "private";

				this.aObjects[x].cNamespace = loType.Namespace;

				if (!llDontParseMethods) 
				{
					this.aObjects[x].LoadMethods(loType);
					this.aObjects[x].LoadProperties(loType);
					this.aObjects[x].LoadEvents(loType);
				}
#if false
				object[] loAttributes = loType.GetCustomAttributes(true);
				if (loAttributes.Length > 0) 
				{
					this.aObjects[x].aAttributes = new string[loAttributes.Length];
					for (int z = 0; z < loAttributes.Length; z++) 
					{
						object loAttribute = loAttributes[z];
						this.aObjects[x].aAttributes[z] = loAttributes[z].ToString();
						int zz = 0;
					}
				}
#endif

			}

			if (this.cXmlFilename.Length > 0)  
			{
				this.ParseXmlProperties(null);
			}
			return lnCount;
		}

		/// <summary>
		/// Retrieves all objects into the aObjects array. Also
		/// parses all the subojects.
		/// </summary>
		/// <returns>Count of objects</returns>
		public int GetAllObjects() 
		{
			return this.GetAllObjects(false);
		}
		
		/// <summary>
		/// This method parses comments from C# XML documentation into the appropriate
		/// cHelpText properties.
		/// </summary>
		/// <returns></returns>
		public bool ParseXmlProperties(DotnetObject loPassedObject) 
		{
			string lcXPath;
			DotnetObject loObject;

			XmlDocument oDom = new XmlDocument();

			try 
			{
				oDom.Load("file://" + this.cXmlFilename);      
			}
			catch(Exception) 
			{
				this.cErrorMsg = "Unable to load XML Documentation file.";
				return false;
			}
			
			// *** Get object descriptions
			for (int lnObjs = 0; lnObjs < this.nObjectCount; lnObjs++) 
			{
				XmlNode loMatch;
				XmlNode loSubMatch;

				if (loPassedObject == null)
					loObject = this.aObjects[lnObjs];
				else 
				{
					loObject = loPassedObject;
					lnObjs = this.nObjectCount - 1;
				}

				if (loPassedObject != null && loPassedObject.cName.ToLower() != loObject.cName.ToLower())
					continue;

				try 
				{
					loMatch = oDom.SelectSingleNode("/doc/members/member[@name='" +
						"T:" + loObject.cNamespace + "." + loObject.cName + "']");

					if (loMatch != null) 
					{
						loSubMatch = loMatch.SelectSingleNode("summary");
						if (loSubMatch != null) 
						{
							loObject.cHelpText = this.FixHelpString(loSubMatch.InnerText.Substring(2));
							loSubMatch = null;
						}
					}
					loMatch = null;
				}
				catch(Exception ex) 
				{ 
					string lcError = ex.Message; 
				}

				// *** Loop through the methods
				for (int lnMethods = 0; lnMethods < loObject.nMethodCount; lnMethods++) 
				{
					ObjectMethod loMethod = loObject.aMethods[lnMethods];

					lcXPath = "/doc/members/member[@name='" +
						"M:" + loObject.cNamespace + "." + loObject.cName + "." + loMethod.cName + 
						"(" + loMethod.cRawParameters + ")']";

					try 
					{
						loMatch = oDom.SelectSingleNode(lcXPath);

						if (loMatch != null) 
						{
							loSubMatch = loMatch.SelectSingleNode("summary");
							if (loSubMatch != null) 
							{
								loMethod.cHelpText = this.FixHelpString(loSubMatch.InnerText.Substring(2));
								loSubMatch = null;
							}

							string lcParmDescriptions = "";
							for (int x = 0; x < loMethod.nParameterCount; x++) 
							{
								lcParmDescriptions = lcParmDescriptions + 
                                      "<<b>>" + loMethod.aParameters[x] + "<</b>>\r\n"; 

								loSubMatch = loMatch.SelectSingleNode("param[@name='" + loMethod.aParameters[x]  +"']");
								if (loSubMatch != null) 
								{
									string lcValue = loSubMatch.InnerText;
									if (lcValue.Length > 0)
										lcParmDescriptions = lcParmDescriptions + lcValue + "\r\n\r\n";
									else
										lcParmDescriptions = lcParmDescriptions + "\r\n";
								}

							}
							loMethod.cDescriptiveParameters = lcParmDescriptions;
						}
						
						loMatch = null;
					}
					catch(Exception ex) 
					{ 
						string lcError = ex.Message; 
					}
				}  // for methods

				// *** Loop through the properties
				for (int lnFields = 0; lnFields < loObject.nPropertyCount; lnFields++) 
				{
					ObjectProperty loProperty = loObject.aProperties[lnFields];
				
					string lcFieldPrefix = "F:";
					if (loProperty.cFieldOrProperty != "Field") 
						lcFieldPrefix = "P:";

					lcXPath = "/doc/members/member[@name='" + lcFieldPrefix +
						loObject.cNamespace + "." + loObject.cName + "." + loProperty.cName + "']";

					try 
					{
						loMatch = oDom.SelectSingleNode(lcXPath);

						if (loMatch != null) 
						{
							loSubMatch = loMatch.SelectSingleNode("summary");
							if (loSubMatch != null) 
							{
								loProperty.cHelpText = this.FixHelpString(loSubMatch.InnerText.Substring(2));
								loSubMatch = null;
							}
						}
						loMatch = null;
					}
					catch(Exception ex) 
					{ 
						string lcError = ex.Message; 
					}
				}  // for properties

				// *** Loop through the properties
				for (int lnFields = 0; lnFields < loObject.nEventCount; lnFields++) 
				{
					ObjectEvent loEvent = loObject.aEvents[lnFields];
				
					string lcFieldPrefix = "E:";

					lcXPath = "/doc/members/member[@name='" + lcFieldPrefix  +
						loObject.cNamespace + "." + loObject.cName + "." + loEvent.cName + "']";

					try 
					{
						loMatch = oDom.SelectSingleNode(lcXPath);

						if (loMatch != null) 
						{
							loSubMatch = loMatch.SelectSingleNode("summary");
							if (loSubMatch != null) 
							{
								loEvent.cHelpText = this.FixHelpString(loSubMatch.InnerText.Substring(2));
								loSubMatch = null;
							}
						}
						loMatch = null;
					}
					catch(Exception ex) 
					{ 
						string lcError = ex.Message; 
					}
				}  // for events

			}  // for objects
            
				return true;
			} // ParseXMLProperties (method)

			
		protected string FixHelpString(string lcString) 
		{
			string[] lcStrings = lcString.Trim().Split((char) 13);
			lcString = "";
			for (int x=0; x < lcStrings.Length; x++) 
			{
				lcString = lcString + lcStrings[x].Trim() + "\r\n";
			}

			return lcString;
		}
	}

	[ClassInterface(ClassInterfaceType.AutoDual)]
	[ProgId("wwReflection.DotNetObject")]
	public class DotnetObject 
	{
		public string cName = "";
		public string cScope = "";
		public string cHelpText = "";
		public string cNamespace = "";
		public string cSyntax = "VB";
		
		public ObjectMethod[] aMethods;
		public int nMethodCount = 0;

		public ObjectProperty[] aProperties;
		public int nPropertyCount = 0;

		public ObjectEvent[] aEvents;
		public int nEventCount = 0;

		//public string[] aAttributes;

		
		//public aConstant[] aConstants;

		/// <summary>
		/// Underlying .Net wwReflection.Type object
		/// </summary>
		//public Type oXType;

		/// <summary>
		/// Loads all methods for a given type into the aMethod array
		/// </summary>
		/// <param name="loType">the type to load methods from</param>
		/// <returns></returns>
		public int  LoadMethods(Type loType) 
		{
			MethodInfo[] loMethods = loType.GetMethods();
			if (loMethods.Length < 1) 
			{
				this.nMethodCount = 0;
				return 0;
			}
			
			int lnCount = loMethods.Length;
			this.nMethodCount = lnCount;

			this.aMethods = new ObjectMethod[lnCount];

			for (int x=0; x < lnCount; x++) 
			{
				MethodInfo loMethod = loMethods[x];
				this.aMethods[x] = new ObjectMethod();

				// *** Skip Property Get and Set methods
//				if (loMethod.Name.Substring(0,4) == "set_" ||
//					loMethod.Name.Substring(0,4) == "get_")
//				   continue;

				ObjectMethod loM = this.aMethods[x];
				loM.cName = loMethod.Name;

				if (loMethod.IsPublic) 
					loM.cScope = "public";
				else if (loMethod.IsPrivate)
					loM.cScope = "private";
				else 
					loM.cScope = "protected";

				loM.cOther = "";
				if (loMethod.IsAbstract)
					loM.cOther = "abstract|";
				if (loMethod.IsVirtual)
					loM.cOther = loM.cOther + "virtual|";
				if (loMethod.IsFinal)
					loM.cOther = loM.cOther + "finalized|";
				if (loMethod.IsStatic)
					loM.cOther = loM.cOther + "static|";

				this.aMethods[x].cReturnType = loMethod.ReturnType.Name;

				ParameterInfo[] loParameters = loMethod.GetParameters();
				this.aMethods[x].nParameterCount = loParameters.Length;

				if (loParameters.Length > 0) 
					this.aMethods[x].aParameters = new string[loParameters.Length];


				string lcParameters = "";
				string lcRawParameters = "";
				
				for (int y = 0; y < loParameters.Length; y++) 
				{
					if (this.cSyntax == "VB") 
						lcParameters = lcParameters + loParameters[y].Name + " as " + loParameters[y].ParameterType.Name;
					else
						lcParameters = lcParameters + loParameters[y].ParameterType.Name + " " + loParameters[y].Name;

					if (y < loParameters.Length - 1) 
						lcParameters= lcParameters + ", ";

					lcRawParameters = lcRawParameters + loParameters[y].ParameterType.FullName;
					if (y < loParameters.Length - 1) 
						lcRawParameters= lcRawParameters + ",";

					this.aMethods[x].aParameters[y] = loParameters[y].Name;
				}
				this.aMethods[x].cParameters = lcParameters;
				this.aMethods[x].cRawParameters = lcRawParameters;
			}

			return lnCount;
		}

		/// <summary>
		/// Loads properties and fields into the aProperties array.
		/// </summary>
		/// <param name="loType">the type to load from</param>
		/// <returns></returns>
		public int  LoadProperties(Type loType) 
		{
			FieldInfo[] loFields = loType.GetFields();
			PropertyInfo[] loProperties = loType.GetProperties();

			int lnCount = loFields.Length + loProperties.Length;
			if (lnCount < 1) 
			{
				return 0;
			}
			this.nPropertyCount = lnCount;
			this.aProperties = new ObjectProperty[lnCount];

			for (int x=0; x < loFields.Length; x++) 
			{
				FieldInfo loProperty = loFields[x];
				this.aProperties[x] = new ObjectProperty();

				ObjectProperty loM = this.aProperties[x];
				loM.cName = loProperty.Name;
				loM.cFieldOrProperty = "Field";
				loM.cType = loProperty.FieldType.Name;

				if (loProperty.IsPublic) 
					loM.cScope = "public";
				else if (loProperty.IsPrivate)
					loM.cScope = "private";
				else 
					loM.cScope = "protected";

				if (loProperty.IsStatic)
					loM.bStatic = true;
			}

			
			if (loProperties.Length < 1)
				return lnCount;
			
			int lnFieldCount = loFields.Length;
			for (int x=0; x < loProperties.Length; x++) 
			{
				PropertyInfo loProperty = loProperties[x];
				this.aProperties[lnFieldCount + x] = new ObjectProperty();

				ObjectProperty loM = this.aProperties[lnFieldCount + x];
				loM.cName = loProperty.Name;
				loM.cFieldOrProperty = "Property";
				loM.cType = loProperty.PropertyType.Name;
				
				loM.cScope = "public";

				if (loProperty.CanRead && loProperty.CanWrite) 
					loM.cAccessType = "";
				else if(loProperty.CanRead)
					loM.cAccessType = "ReadOnly";
				else if(loProperty.CanWrite)
                    loM.cAccessType = "WriteOnly";
			}

			return lnCount;
		}


		/// <summary>
		/// Load Events from an object into aEvents property
		/// </summary>
		/// <param name="loType">the type to load from</param>
		/// <returns></returns>
		public int  LoadEvents(Type loType)  
		{
			EventInfo[] loFields = loType.GetEvents();
			
			int lnCount = loFields.Length ;
			if (lnCount < 1) 
			{
				return 0;
			}

			this.nEventCount = lnCount;
			this.aEvents = new ObjectEvent[lnCount];

			for (int x=0; x < loFields.Length; x++) 
			{
				EventInfo loEvent= loFields[x];
				this.aEvents[x] = new ObjectEvent();

				ObjectEvent loM = this.aEvents[x];
				loM.cName = loEvent.Name;
				loM.cType = loEvent.EventHandlerType.ToString();

				loM.cScope = "public";
			}

			return lnCount;
		}


	}

	[ClassInterface(ClassInterfaceType.AutoDual)]
	[ProgId("wwReflection.ObjectMethod")]
	public class ObjectMethod 
	{
		public string cName = "";
		public string cParameters = "";
		public string cRawParameters = "";
		public string cDescriptiveParameters = "";
		public string cReturnType = "";
		public string cHelpText = "";
		public string cScope = "";
		public bool bStatice = false;
		public string cOther = "";
		public string[] aParameters = null;
		public int nParameterCount = 0;

	}

	[ClassInterface(ClassInterfaceType.AutoDual)]
	[ProgId("wwReflection.ObjectProperty")]
	public class ObjectProperty
	{
		public string cName = "";
		public string cType = "";
        		
		public string cScope = "";
		public string cAccessType = "";
		public string cDefaultValue = "";

		public bool bStatic = false;

		public string cOther = "";
		
		/// <summary>
		/// Field or Property
		/// </summary>
		public string cFieldOrProperty = "";

		/// <summary>
		/// Not implemented
		/// </summary>
		public string cHelpText = "";

	}

	public class ObjectEvent 
	{
		public string cName = "";
		public string cType = "";
        		
		public string cScope = "";
		public bool bStatic = false;

		public string cOther = "";
		
		public string cHelpText = "";

	}


}
