using System;
using Westwind.wwReflection.Tools;

namespace Westwind.wwHelp
{
	/// <summary>
	/// The Config class maps to the various Help Builder IDE configuration 
	/// settings. These settings map to the settings found in wwHelp.ini.
	/// 
	/// In addition to the properties on this object you can also dynamically 
	/// access any values via the Indexer:
	/// 
	/// string s = (string) Help.Config[&quot;ClassOpenPath&quot;];
	/// </summary>
	public class wwHelpConfig : wwHelpComWrapper
	{
		public wwHelpConfig(object ComInstance) 
		{
			this.m_Inst = ComInstance;
		}

		public wwHelpConfig()
		{
		}

		/// <summary>
		/// Indexer for the Config object to retrieve any non property values
		/// that are part of this interface. If the value is not found 
		/// null is returned.
		/// </summary>
		public object this[string index] 
		{
			get
			{
				return wwUtils.CallMethodCom(this.m_Inst,"GetValue",index) ;
			}
			set
			{
				wwUtils.CallMethodCom(this.m_Inst,"SetValue",index,value);
			}
		}

		/// <summary>
		/// Loads the configuration file from disk
		/// </summary>
		/// <returns></returns>
		public bool Load() 
		{
			return (bool) wwUtils.CallMethodCom(this.m_Inst,"Load",false);
		}

		/// <summary>
		/// Saves the COnfiguration file to disk from the current settings.
		/// </summary>
		/// <returns></returns>
		public bool Save() 
		{
			return (bool) wwUtils.CallMethodCom(this.m_Inst,"Save",false);
		}
	
		/// <summary>
		/// The last file that was used before this current one.
		/// </summary>
		public string LastFile
		{
			get { return (string) wwUtils.GetPropertyCom(m_Inst,"cLastFile"); }
			set { wwUtils.SetPropertyCom(m_Inst,"cLastFile",value); }
		}
		
		/// <summary>
		/// Pk of the last topic accessed
		/// </summary>
		public string LastTopic
		{
			get { return (string) wwUtils.GetPropertyCom(m_Inst,"cLastTopic"); }
			set { wwUtils.SetPropertyCom(m_Inst,"cLastTopic",value); }
		}

		public string EditFont
		{
			get { return (string) wwUtils.GetPropertyCom(m_Inst,"cEditFont"); }
			set { wwUtils.SetPropertyCom(m_Inst,"cEditFont",value); }
		}
		public string CodeFont
		{
			get { return (string) wwUtils.GetPropertyCom(m_Inst,"cCodeFont"); }
			set { wwUtils.SetPropertyCom(m_Inst,"cCodeFont",value); }
		}
		public bool TabsOnBottom
		{
			get { return (bool) wwUtils.GetPropertyCom(m_Inst,"lTabsOnBottom"); }
			set { wwUtils.SetPropertyCom(m_Inst,"lTabsOnBottom",value); }
		}

		/// <summary>
		/// The editor used to edit HTML Templates and text/CSS.
		/// </summary>
		public string TemplateEditor
		{
			get { return (string) wwUtils.GetPropertyCom(m_Inst,"cTemplateEditor"); }
			set { wwUtils.SetPropertyCom(m_Inst,"cTemplateEditor",value); }
		}

		/// <summary>
		/// Saved value for the last import from a .NET component that keeps track
		/// to see if XML comments are word wrapped.
		/// </summary>
		public bool WordWrapImports
		{
			get { return (bool) wwUtils.GetPropertyCom(m_Inst,"lWordWrapImports"); }
			set { wwUtils.SetPropertyCom(m_Inst,"lWordWrapImports",value); }
		}

		/// <summary>
		/// The .NET language used on the last import.
		/// </summary>
		public string DotNetImportLanguage
		{
			get { return (string) wwUtils.GetPropertyCom(m_Inst,"cDotNetLanguageImports"); }
			set { wwUtils.SetPropertyCom(m_Inst,"cDotNetLanguageImports",value); }
		}
	
		/// <summary>
		/// List of recently used topics separated by |
		/// </summary>
		public string RecentList
		{
			get { return (string) wwUtils.GetPropertyCom(m_Inst,"cRecentList"); }
			set { wwUtils.SetPropertyCom(m_Inst,"cRecentList",value); }
		}

		/// <summary>
		/// Maximum number of recent items to display
		/// </summary>
		public int MaxRecent
		{
			get { return (int) Convert.ToInt32(wwUtils.GetPropertyCom(m_Inst,"nMaxRecent")); }
			set { wwUtils.SetPropertyCom(m_Inst,"nMaxRecent",value); }
		}

		public DateTime LastVersionCheck
		{
			get { return (DateTime) wwUtils.GetPropertyCom(m_Inst,"dLastVersionCheck"); }
			set { wwUtils.SetPropertyCom(m_Inst,"dLastVersionCheck",value); }
		}

		public string SnagItOptions
		{
			get { return (string) wwUtils.GetPropertyCom(m_Inst,"cSnagItOptions"); }
			set { wwUtils.SetPropertyCom(m_Inst,"cSnagItOptions",value); }
		}
		public bool UseSnagItForScreenCapture
		{
			get { return (bool) wwUtils.GetPropertyCom(m_Inst,"lUseSnagItForScreenCapture"); }
			set { wwUtils.SetPropertyCom(m_Inst,"lUseSnagItForScreenCapture",value); }
		}

		/// <summary>
		/// The default FTP Server used for uploads.
		/// </summary>
		public string FtpServer
		{
			get { return (string) wwUtils.GetPropertyCom(m_Inst,"cFtpServer"); }
			set { wwUtils.SetPropertyCom(m_Inst,"cFtpServer",value); }
		}

		/// <summary>
		/// The default username used for FTP uploads
		/// </summary>
		public string FtpUsername
		{
			get { return (string) wwUtils.GetPropertyCom(m_Inst,"cFtpUsername"); }
			set { wwUtils.SetPropertyCom(m_Inst,"cFtpUsername",value); }
		}

	}		

}
