*&* LOG4FOX.PRG
*&* An interpretation of the LOG4J API for Visual FoxPro
*&* (c) Lisa Slater Nicholls

#INCLUDE LOG4FOX.H


DEFINE CLASS Log4FoxConfigAlias AS Log4Fox

   PROTECTED PROCEDURE Init(oOwner, lOwnerNotRequired, vConfiguration) AS Boolean
      IF DODEFAULT(oOwner,lOwnerNotRequired, vConfiguration)
         IF ISNULL(THIS.Configuration)
            RETURN .F.
         ENDIF
      ELSE
         RETURN .F.
      ENDIF
   ENDPROC
   
   PROCEDURE Configuration_Assign(val)
      IF VARTYPE(val) = "C" AND USED(val) 
         * good to go, we
         * want to be able to use whatever
         * values belong to this external object
         * and also pass on the alias availability
         * to the other members of the group
         IF ISNULL(THIS.Appenders) 
            THIS.Appenders = CREATEOBJECT("Collection")
         ENDIF
         IF TYPE(val + ".logger") = "C"
            * try to create any that exist
            * so they can be configured right off
            * some can't be, they need extra params, 
            * but the extra params *may* be in 
            * other fields of the table that they know about
            LOCAL liSelect, loX
            liSelect = SELECT()
            SELECT (val)
            SCAN FOR NOT EMPTY(logger)
                TRY
                   loX = CREATEOBJECT(ALLTRIM(logger), THIS)
                CATCH WHEN .T.
                ENDTRY
            ENDSCAN
            SELECT (liSelect)
         ENDIF
         IF TYPE(val+".autoOpen") = "L" AND ;
            EVALUATE(val + ".autoOpen")
            THIS.Open()
         ENDIF
         IF TYPE(val+".logLevel") = "N" 
            GO TOP IN (val)
            * global setting; 
            * additional records might
            * have additional information
            * for specific appenders
            THIS.SetLevel(EVALUATE(val + ".logLevel"))
         ENDIF
      ELSE
         val = NULL
      ENDIF   
      DODEFAULT(val)
   ENDPROC

ENDDEFINE


DEFINE CLASS Log4Fox AS VFPHelperObject 

   Appenders = NULL
   Configuration = NULL
   
   PROTECTED PROCEDURE Init(oOwner, lOwnerNotRequired, vConfiguration) AS Boolean
      IF DODEFAULT(oOwner,lOwnerNotRequired) 
         THIS.Configuration = vConfiguration
         IF ISNULL(THIS.Appenders)
            * configuration object may have
            * created and populated it
            THIS.Appenders = CREATEOBJECT("Collection")         
         ENDIF            
      ELSE
        RETURN .F.
      ENDIF
   ENDPROC
   
   PROCEDURE Configuration_Assign(val)
      * validate for type and appropriateness
      * in subclasses
      THIS.Configuration = val
      THIS.ErrorMessage = ""
      * If appenders available,
      * use configuration information to 
      * set switches and options
      * (none provided in this baseclass factory object)
      * and provide to Appenders:
      IF VARTYPE(THIS.Appenders) = "O"
         LOCAL appender
         FOR EACH appender IN THIS.Appenders FOXOBJECT
             appender.SetConfiguration()
         ENDFOR         
      ENDIF            
   ENDPROC      

   PROCEDURE Log(whatever,loglevel,logevent)
      IF VARTYPE(THIS.Appenders) = "O" 
         LOCAL appender
         THIS.ErrorMessage = ""
         FOR EACH appender IN THIS.Appenders FOXOBJECT
             appender.DoLog(whatever,loglevel,logevent)
         ENDFOR
      ENDIF         
   ENDPROC
   
   PROCEDURE Open()
      IF VARTYPE(THIS.Appenders) = "O"
         LOCAL appender
         THIS.ErrorMessage = ""
         FOR EACH appender IN THIS.Appenders FOXOBJECT
             appender.OpenLog()
         ENDFOR
      ENDIF
   ENDPROC
   
   PROCEDURE Close()
      IF VARTYPE(THIS.Appenders) = "O"
         LOCAL appender
         THIS.ErrorMessage = ""
         FOR EACH appender IN THIS.Appenders FOXOBJECT
            appender.CloseLog()
         ENDFOR
      ENDIF         
   ENDPROC
   
   PROCEDURE SetLevel(iLevel)
      IF VARTYPE(THIS.Appenders) = "O"
         LOCAL appender
         THIS.ErrorMessage = ""
         FOR EACH appender IN THIS.Appenders FOXOBJECT
             appender.SetLogLevel(iLevel)
         ENDFOR
      ENDIF         
   ENDPROC
   
   PROTECTED PROCEDURE Release
      IF VARTYPE(THIS.Appenders) = "O"
         THIS.Appenders.Remove(-1)
         THIS.Appenders = NULL
      ENDIF         
      DODEFAULT()
   ENDPROC

ENDDEFINE

DEFINE Class log4FoxSQLAppender AS log4FoxAbstractAppender
   PROTECTED CurrentLog
   LogHandle = -1
   LogRecord = "DefaultLog"
   CurrentLog = ""
   
   * This simple version assumes an available SQL table with the following structure
*&*	   CREATE TABLE [EventLog] (
*&*	      [logger] [varchar] (15) NOT NULL,
*&*	      [create_date] [datetime] NULL ,
*&*	      [log_events] [text]  NULL ,
*&*	      CONSTRAINT [PK_EventLog] PRIMARY KEY  CLUSTERED
*&*	      (
*&*	         [logger]
*&*	      )  ON [PRIMARY]
*&*	      ) ON [PRIMARY] TEXTIMAGE_ON [PRIMARY]
*&*	    GO

   
   PROTECTED PROCEDURE SendLog(whatever)
      * build up contents whether log is open or not
      LOCAL logString
      logString = STRTRAN(whatever, "'","")      
      THIS.CurrentLog = THIS.CurrentLog + CHR(13) + logString
   ENDPROC
   
   PROTECTED PROCEDURE SetLogParameters(LoggerName, overWrite)
       * read from configuration info here
       IF VARTYPE(THIS.Configuration) = "C"
          LOCAL liSelect
          liSelect = SELECT()
          SELECT (THIS.Configuration)
          IF TYPE(THIS.Configuration + ".Logger") = "C"
             * try to find a record for this class
             * or use global settings
             LOCATE FOR ALLTRIM(UPPER(Logger)) == UPPER(THIS.Class)
             IF EOF()
                GO TOP 
             ENDIF
          ELSE
             GO TOP 
          ENDIF
          IF TYPE("LogName") = "C" && could be var *or* field
             LoggerName = LogName
          ENDIF
          IF TYPE("LogLevel") = "N" && could be var *or* field
             THIS.SetLogLevel(LogLevel)
          ENDIF
          IF TYPE("LogAppend") = "L" && could be var *or* field
             overWrite = (NOT LogAppend)
          ENDIF
          SELECT (liSelect)   
       ELSE
          * you could do other stuff here if you want,
          * such as reading configuration information
          * from your SQL database, obviously!   
      ENDIF            
   ENDPROC

   PROTECTED PROCEDURE ValidateConfiguration(vConfiguration)
      * this version will understand only
      * "alias-type" configuration data ...
      RETURN (ISNULL(vConfiguration)) OR ;
             (EMPTY(vConfiguration)) OR ;
             (VARTYPE(vConfiguration) = "C" AND USED(vConfiguration))
   ENDPROC
      
   PROCEDURE OpenLog()
      LOCAL LoggerName, overWrite
      THIS.SetLogParameters(@LoggerName, @overWrite)
      IF THIS.LogHandle = -1
         THIS.LogOpen = .F.
         RETURN .F.
      ENDIF    
      IF EMPTY(LoggerName) AND THIS.LogOpen
         RETURN .T.
      ENDIF
      THIS.CloseLog()
      LOCAL thislog, logAlias, curSelect
      logAlias = "L"+SYS(2015)
      curSelect = SELECT()
      IF VARTYPE(LoggerName)="C" AND NOT EMPTY(LoggerName)
         thislog = LoggerName
      ELSE
         thislog = "log_"+SYS(2015)
      ENDIF
      THIS.ErrorMessage = ""
      IF SQLEXEC(THIS.LogHandle, ;
        "SELECT * FROM EventLog WHERE logger = '"+thislog+"'",logalias) > 0 
         IF USED(logAlias) 
            IF RECCOUNT(logAlias) = 0 
               THIS.CurrentLog = ""
               SQLEXEC(THIS.LogHandle, ;
                     "INSERT INTO EventLog " + ;
                     "(create_date, logger) VALUES " + ;
                     "('"+TTOC(DATETIME())+"',"+ ;
                     "'"+thislog+"')")      
            ELSE
               IF overWrite
                  THIS.CurrentLog = ""            
                  SQLEXEC(THIS.LogHandle, ;
                     "UPDATE EventLog " + ;
                     "SET create_date = '"+TTOC(DATETIME())+"',"+ ;
                     " log_events=NULL WHERE logger = '"+thislog+"'")      
               ELSE
                  SELECT (logAlias)            
                  THIS.CurrentLog = NVL(log_events,"")
               ENDIF
            ENDIF
             
         ENDIF
      ELSE
         THIS.ErrorMessage = THIS.Class +":" + MESSAGE()
      ENDIF
      IF EMPTY(THIS.ErrorMessage)
         THIS.LogRecord = thislog
         THIS.LogOpen = .T.
         THIS.DoLog(THIS.GetStandardLogOpenedMessage(),LOGLEVEL_INFO)
      ELSE
         THIS.LogOpen = .F.
         THIS.LogHandle = -1
      ENDIF
      IF USED(logAlias)
         USE IN (logAlias)
      ENDIF
      SELECT (curSelect)
      RETURN (THIS.LogOpen)
   ENDPROC

   PROCEDURE CloseLog()
       THIS.DoLog(THIS.GetStandardLogClosedMessage(),LOGLEVEL_INFO)
       IF THIS.LogOpen AND ;
          LEN(THIS.CurrentLog) > 0
          SQLEXEC(THIS.LogHandle, ;
                  "UPDATE EventLog " + ;
                  "SET log_events='"+ THIS.CurrentLog+"' "+ ;
                  "WHERE logger = '"+THIS.LogRecord+"'")      
       ENDIF
       THIS.CurrentLog = ""
       DODEFAULT()
   ENDPROC
    
   PROTECTED PROCEDURE GetCurrentLogContents() 
     LOCAL theContents
     theContents = ""
     *&* doesn't matter whether at this point THIS.LogOpen  
     THIS.DoLog("Asked for current log contents...",LOGLEVEL_DEBUG)
     theContents = THIS.CurrentLog
   RETURN theContents
   
   PROTECTED PROCEDURE Init(oOwner, lOwnerNotRequired, iHandle)   
      THIS.LogHandle = VAL(TRANSFORM(iHandle))
      RETURN (THIS.LogHandle > 0) AND DODEFAULT(oOwner, lOwnerNotRequired)
   ENDPROC
   

ENDDEFINE

DEFINE CLASS log4FoxFileAppender AS log4FoxAbstractAppender
      
   PROTECTED LogFile
   LogFile = NULL   
   
   PROTECTED PROCEDURE SetLogParameters(fileName, overWrite)
      * could read from configuration here to set filename,
      * overwrite preference, loglevel (either globally
      * or on a local level, this-appender-only)
      IF NOT ISNULL(THIS.LogFile)
         fileName= THIS.LogFile
      ENDIF         
   ENDPROC
   
   PROCEDURE OpenLog() 
      LOCAL fileName, overWrite
      THIS.SetLogParameters(@fileName, @overWrite)
      IF EMPTY(fileName) AND THIS.LogOpen
         RETURN .T.
      ENDIF
      LOCAL thislog
      IF VARTYPE(fileName)="C" AND NOT EMPTY(fileName)
         thislog = fileName
      ELSE
         thislog = "log_"+SYS(2015)
      ENDIF
      IF EMPTY(JUSTEXT(thislog))
         thislog = FORCEEXT(thislog,"LOG")
      ENDIF
      THIS.ErrorMessage = ""
      IF THIS.LogOpen AND ;
         ((filename # thislog) OR ;
          overWrite)        
         THIS.CloseLog()
      ENDIF
      IF overWrite
         IF FILE(thislog)
            ERASE (thislog) RECYCLE
         ENDIF
         SET TEXTMERGE TO (thislog)  NOSHOW
      ELSE
         SET TEXTMERGE TO (thislog) ADDITIVE NOSHOW
      ENDIF 
      SET TEXTMERGE ON  
      IF EMPTY(THIS.ErrorMessage)
         THIS.LogFile = thislog
         THIS.LogOpen = .T.
         THIS.DoLog(THIS.GetStandardLogOpenedMessage(),LOGLEVEL_INFO)
      ELSE
         THIS.LogOpen = .F.
         THIS.CloseLog()
      ENDIF
   RETURN (THIS.LogOpen)
     
   PROCEDURE CloseLog()
       THIS.DoLog(THIS.GetStandardLogClosedMessage(),LOGLEVEL_INFO)
       SET TEXTMERGE TO
       SET TEXTMERGE OFF
       DODEFAULT()
   ENDPROC
   
   PROTECTED PROCEDURE SendLog(whatever)
      IF THIS.LogOpen
         \<<whatever>>
      ENDIF
   ENDPROC   
   
   PROTECTED PROCEDURE GetCurrentLogContents() 
     LOCAL theContents
     theContents = ""
     IF THIS.LogOpen
        THIS.DoLog("Asked for current log contents...",LOGLEVEL_DEBUG)
        THIS.CloseLog()
        IF FILE(THIS.LogFile)
           theContents = FILETOSTR(THIS.LogFile)
        ENDIF
        THIS.OpenLog()
     ENDIF
   RETURN theContents
   
   PROCEDURE GetLogFileName()
      RETURN THIS.LogFile
   ENDPROC
   
   PROCEDURE SetLogFileName(val)
      IF VARTYPE(val) = "C" AND (NOT EMPTY(val)) AND ;
         (NOT THIS.LogOpen)
         THIS.LogFile = val
      ENDIF
   ENDPROC
   
      
ENDDEFINE

DEFINE CLASS log4FoxMessageBoxAppender AS log4FoxAbstractAppender
    PROTECTED CurrentIcon
    CurrentIcon = 0
    Timeout = 3000
    Caption = "log4FoxMessageBoxAppender"
    
   PROCEDURE OpenLog()
      THIS.Layout.IncludeDateTimeStamp = .F.
      THIS.Layout.IncludeLogEvent = .F.
      DODEFAULT()
   ENDPROC
   
   PROCEDURE DoLog(whatever, loglevel,logevent)
      LOCAL liIcon
      liIcon = VAL(TRANSFORM(logLevel))
      DO CASE
      CASE liIcon < LOGLEVEL_WARN 
         THIS.CurrentIcon = 16 && stop
      CASE liIcon < LOGLEVEL_INFO
         THIS.CurrentIcon = 48 && excl
      CASE liIcon < LOGLEVEL_DEBUG
         THIS.CurrentIcon = 64 && info
      OTHERWISE
         THIS.CurrentIcon = 0
      ENDCASE                  
      DODEFAULT(whatever, loglevel, logevent)
   ENDPROC   


   PROTECTED PROCEDURE SendLog(whatever)
      MESSAGEBOX(whatever,THIS.CurrentIcon,THIS.Caption, THIS.Timeout)
   ENDPROC


ENDDEFINE

DEFINE CLASS log4FoxConsoleAppender AS log4FoxAbstractAppender
   
   PROCEDURE OpenLog()       
      IF NOT WVISIBLE("Debug Output")
         LOCAL lcWindow
         lcWindow = WOUTPUT()
         ACTI WINDOW "DEBUG OUTPUT"
         IF EMPTY(lcWindow)
            ACTI SCREEN
         ELSE
            ACTI WINDOW (lcWindow)
         ENDIF
      ENDIF
      IF WVISIBLE("Debug Output")
         THIS.LogOpen = .T.
         THIS.DoLog(THIS.GetStandardLogOpenedMessage(),LOGLEVEL_INFO)
      ELSE
         THIS.LogOpen = .F.
      ENDIF
      THIS.ErrorMessage = ""
   RETURN (THIS.LogOpen)
     
   PROCEDURE CloseLog()
       THIS.DoLog(THIS.GetStandardLogClosedMessage(),LOGLEVEL_INFO)
       DODEFAULT()
   ENDPROC
   
   PROTECTED PROCEDURE SendLog(whatever)
      IF THIS.LogOpen
         DEBUGOUT whatever
      ENDIF
   ENDPROC
         
ENDDEFINE


DEFINE CLASS log4FoxAbstractAppender AS VFPHelperObject 

*&*	   this class provides an abstract logging API
*&*	   
*&*	   As a descendent of VFPHelperObject, 
*&*	   it defers error-handling to its owner and
*&*	   will not instantiate if its owner does not exist.   
*&*	   However since VFPHelperObject also has an ErrorMessage
*&*	   property it can handle log-opening
*&*	   errors directly, before deferring error-handling
*&*	   to the owner.
      
   PROTECTED LogLevel, LogOpen, Layout, Configuration, MyKey
   LogLevel = LOGLEVEL_FATAL  
   LogOpen = .F.
   Layout = NULL
   Configuration = NULL
   MyKey = SYS(2015)
   
   PROTECTED PROCEDURE Init(oOwner,lOwnerNotRequired)
      IF DODEFAULT(oOwner) AND ;
         TYPE("oOwner.Appenders") = "O" AND ;
         UPPER(oOwner.Appenders.BaseClass) == "COLLECTION"  AND ;
         (NOT (UPPER(THIS.Class) == "LOG4FOXABSTRACTAPPENDER")) 
         * this class has constraints:
         * must be owned by a proper owner
         * must not be instantiated directly (abstract)
         THIS.SetConfiguration()
         IF THIS.HasValidFormatter() OR THIS.SetFormatter() 
            THIS.MyKey = SYS(2015)
            THIS.Owner.Appenders.Add(THIS,THIS.MyKey)
         ELSE
            RETURN .F.
         ENDIF            
      ELSE
         RETURN .F.
      ENDIF                
   ENDPROC
   
   PROCEDURE SetConfiguration(vConfiguration)
      * Appenders have access to their
      * THIS.Owner.Configuration... 
      IF EMPTY(vConfiguration) OR ISNULL(vConfiguration)
         vConfiguration = THIS.Owner.Configuration
      ENDIF
      IF THIS.ValidateConfiguration(vConfiguration)
         * ... and can choose to use it themselves...
         THIS.Configuration = vConfiguration
         * If the configuration is valid,
         * along with setting any switches on the class here,
         * derived classes may have chosen to null out their
         * Layout object based on their configuration information,
         * So re-get at this point.
         
         DO CASE
         CASE THIS.SetFormatter() 
            * ... and they can pass it on 
            * to their Layout objects,
            * although Layout objects are
            * not obligated to fulfill this part 
            * of the API.
            IF PEMSTATUS(THIS.Layout,"Configuration",5)
               THIS.Layout.Configuration = vConfiguration
            ENDIF
         CASE THIS.HasValidFormatter()
            * no change
         OTHERWISE
            RELEASE THIS               
         ENDCASE
      ELSE
         RELEASE THIS
      ENDIF
   ENDPROC
   
   PROTECTED PROCEDURE SetFormatter()
      IF ISNULL(THIS.Layout)
         THIS.GetFormatter()
         IF NOT VARTYPE(THIS.Layout) = "O"
            THIS.GetDefaultFormatter()
         ENDIF
         RETURN THIS.HasValidFormatter()
      ELSE
         RETURN .F.
      ENDIF         
   ENDPROC      
   
   PROTECTED PROCEDURE HasValidFormatter
      * can be augmented if you wish (for example)
      * to require a configuration device
      RETURN ((VARTYPE(THIS.Layout) = "O") AND ;
             PEMSTATUS(THIS.Layout,"StringFormat",5))      
   ENDPROC  
              
   PROTECTED PROCEDURE ValidateConfiguration(vConfiguration)  
      * abstract
   ENDPROC
   
   PROTECTED PROCEDURE GetFormatter()
      * abstract
      * can provide a more sophisticated 
      * layout object than the base.
   ENDPROC
   
   PROTECTED PROCEDURE GetDefaultFormatter()
      * Provides a base implementation,
      THIS.Layout = NEWOBJECT("log4FoxBaseFormatter","","", THIS)
   ENDPROC  
   
   PROTECTED PROCEDURE Release
      * will be augmented in many descendents
      THIS.CloseLog()
      THIS.Layout = NULL
      THIS.Configuration = NULL
      TRY
         THIS.Owner.Appenders.Remove(THIS.MyKey)
      CATCH WHEN .T.
      ENDTRY
      DODEFAULT()
   ENDPROC   
   
   PROTECTED PROCEDURE SendLog(whatever)
     * abstract
   ENDPROC    
    
   PROCEDURE OpenLog()
     * abstract
   ENDPROC
   
   PROTECTED PROCEDURE SetLogParameters()
      * abstract, used in OpenLog,
      * will require overloads in derived classes
   ENDPROC
   
   PROCEDURE CloseLog()
     THIS.LogOpen = .F.
     * abstract, should not require overloads when
     * augmented in descendents
     * and should do no harm if called when the 
     * log is not actually open
   ENDPROC      
   
   PROTECTED PROCEDURE GetStandardLogOpenedMessage()
      RETURN "Log opened, configured to log at level " + ;
                    TRANSFORM(THIS.logLevel)+ "."
   ENDPROC
   
   PROTECTED PROCEDURE GetStandardLogClosedMessage()
      RETURN "Log closed."
   ENDPROC
   
   PROCEDURE GetLogLevel() 
   RETURN THIS.logLevel

   PROCEDURE SetLogLevel(logLevel)
     LOCAL thisLevel
     thisLevel = INT(VAL(TRANSFORM(loglevel)))
     THIS.DoLog("Log level for " + THIS.Name + ;
                 " set at "+TRANSFORM(thisLevel)+".", LOGLEVEL_INFO)
     THIS.loglevel = thislevel
   ENDPROC
      
   PROCEDURE DoLog(whatever, loglevel,logevent)
      LOCAL thislevel
      thislevel = IIF(VARTYPE(loglevel) = "N",loglevel,0)
      IF (NOT ISNULL(THIS.Layout)) AND THIS.LogLevel >= thislevel
          THIS.SendLog(THIS.Layout.StringFormat(whatever,loglevel,logevent))
      ENDIF
   ENDPROC   
   
   PROTECTED PROCEDURE GetCurrentLogContents() AS String
      * abstract, not all types of appenders will be able
      * to fulfill this one, and it is largely for
      * debugging purposes
      RETURN ""
   ENDPROC
   
ENDDEFINE

DEFINE CLASS log4FoxStripHTMLFormatter AS log4FoxBaseFormatter

   PROTECTED RegExpr
   
   RetainHTMLMarkup = .F.
   RegExpr = NULL

   PROTECTED PROCEDURE CheckHTMLMarkup(logString)
      #DEFINE StripHTMLPattern "<[^>]+>"
      IF NOT THIS.RetainHTMLMarkup
         IF ISNULL(THIS.RegExpr)
            THIS.RegExpr = CREATEOBJECT("VBScript.RegExp")
            THIS.RegExpr.Pattern = StripHTMLPattern
            THIS.RegExpr.Global = .T.
            THIS.RegExpr.MultiLine = .T.
         ENDIF
         logString = THIS.RegExpr.Replace(logString," ")
      ENDIF
      RETURN logString
   ENDPROC
   
   PROTECTED PROCEDURE StringFormatHook(logString, loglevel, logevent)
      LOCAL lcString
      lcString = THIS.CheckHTMLMarkup(logString)
      lcString = DODEFAULT(lcString, loglevel, logevent)
      RETURN lcString
   ENDPROC
   
   PROTECTED PROCEDURE Release
      THIS.RegExpr = NULL
      DODEFAULT()
   ENDPROC

ENDDEFINE

DEFINE CLASS log4FoxBaseFormatter AS VFPHelperObject
   
   IncludeLogEvent = .T.
   IncludeLogLevel = .T.
   IncludeDateTimeStamp = .T.
   Configuration = NULL

   PROCEDURE StringFormat(whatever, loglevel, logevent)
      LOCAL lcString
      lcString = TRANSFORM(whatever)
      lcString = THIS.StringFormatHook(lcString, loglevel, logevent)
      RETURN lcString
   ENDPROC
   
   PROTECTED PROCEDURE StringFormatHook(logString, loglevel, logevent)
      LOCAL lcString
      lcString = logString
      IF THIS.IncludeDateTimeStamp
         lcString =  TRANSFORM(DATETIME())+" " + lcString
      ENDIF
      IF THIS.IncludeLogLevel AND VARTYPE(loglevel) = "N"
         lcString = lcString + " (level=" + TRANSFORM(loglevel) + ")"
      ENDIF
      IF THIS.IncludeLogEvent AND NOT EMPTY(logevent)
         lcString = TRANSFORM(logevent)+"_Event: "+ lcString
      ENDIF
      RETURN lcString
   ENDPROC   

   
   PROCEDURE Configuration_Assign(val)
      * validate for type and appropriateness
      IF .T.
         THIS.Configuration = val
         * If valid, use configuration information to 
         * set switches and options
         * (in this class, IncludeLogEvent, IncludeLogLevel, RetainHTMLMarkup)
      ELSE
         THIS.Configuration = NULL         
      ENDIF         
   ENDPROC
   

ENDDEFINE


DEFINE CLASS VFPHelperObject AS Custom
   PROTECTED Owner, ErrorMessage
   Owner = NULL
   ErrorMessage = ""
    
   PROTECTED PROCEDURE Init(oOwner, lOwnerNotRequired)
      THIS.Owner = oOwner
   RETURN (VARTYPE(lOwnerNotRequired) = "L" AND lOwnerNotRequired) OR ;
          (VARTYPE(THIS.Owner) = "O" )
   
   PROCEDURE ERROR(nError, cMethod, nLine)
      THIS.ErrorMessage = MESSAGE() + " (" + cMethod + ") "
      IF (VARTYPE(THIS.Owner) = "O")
         THIS.Owner.Error(nError,THIS.Class + "." + cMethod,nLine)
      ELSE
         ERROR nError
         * ordinarily this should only occur during
         * debugging   
      ENDIF         
   ENDPROC
   
   PROCEDURE GetErrorMessage()
      RETURN THIS.ErrorMessage
   ENDPROC
   
   PROTECTED PROCEDURE Destroy
      THIS.Release()
   ENDPROC
   
   PROTECTED PROCEDURE Release
      THIS.Owner = NULL
   ENDPROC
   
   PROCEDURE TestError
      LOCAL xxx
      xxx = "DO FileDoesNotExist.prg"
      &xxx.
   ENDPROC

ENDDEFINE


